#!/usr/bin/env python2

import espressopp

nsteps      = 100
isteps      = 100
rc          = pow(2.0, 1.0/6.0)
skin        = 0.4
timestep    = 0.005

# set thermostat properties
temperature = 1.0
gamma = 1.0

######################################################################
### IT SHOULD BE UNNECESSARY TO MAKE MODIFICATIONS BELOW THIS LINE ###
######################################################################
print 'Setting up simulation ...'
bonds, angles, x, y, z, Lx, Ly, Lz = espressopp.tools.lammps.read('polymer_melt.lammps')
num_particles = len(x)
density = num_particles / (Lx * Ly * Lz)
size = (Lx, Ly, Lz)
#system, integrator = espressopp.standard_system.Default(box=box, rc=rc, skin=skin, dt=timestep, temperature=temperature)
system = espressopp.System()
system.rng = espressopp.esutil.RNG()
system.bc = espressopp.bc.OrthorhombicBC(system.rng, size)
system.skin = skin


comm = espressopp.MPI.COMM_WORLD
nodeGrid = espressopp.tools.decomp.nodeGrid(comm.size)
cellGrid = espressopp.tools.decomp.cellGrid(size, nodeGrid, rc, skin)

# domain decomposition
system.storage = espressopp.storage.DomainDecomposition(system, nodeGrid, cellGrid)

# integrator
integrator = espressopp.integrator.VelocityVerlet(system)
integrator.dt = timestep

# langevin thermostat
langevin = espressopp.integrator.LangevinThermostat(system)
langevin.gamma = gamma
langevin.temperature = temperature
integrator.addExtension(langevin)


# add particles to the system and then decompose
props = ['id', 'type', 'mass', 'pos']
new_particles = []
for i in range(num_particles):
  part = [i + 1, 0, 1.0, espressopp.Real3D(x[i], y[i], z[i])]
  new_particles.append(part)

system.storage.addParticles(new_particles, *props)
system.storage.decompose()

# Lennard-Jones with Verlet list
vl      = espressopp.VerletList(system, cutoff = rc)
potLJ   = espressopp.interaction.LennardJones(epsilon=1.0, sigma=1.0, cutoff=rc, shift=0)
interLJ = espressopp.interaction.VerletListLennardJones(vl)
interLJ.setPotential(type1=0, type2=0, potential=potLJ)
system.addInteraction(interLJ)

# FENE bonds
fpl = espressopp.FixedPairList(system.storage)
fpl.addBonds(bonds)
potFENE = espressopp.interaction.FENE(K=30.0, r0=0.0, rMax=1.5)
interFENE = espressopp.interaction.FixedPairListFENE(system, fpl, potFENE)
system.addInteraction(interFENE)

# Cosine with FixedTriple list
ftl = espressopp.FixedTripleList(system.storage)
ftl.addTriples(angles)
potCosine = espressopp.interaction.Cosine(K=1.5, theta0=3.1415926)
interCosine = espressopp.interaction.FixedTripleListCosine(system, ftl, potCosine)
system.addInteraction(interCosine)

# print simulation parameters
print ''
print 'number of particles = ', num_particles
print 'density             = ', density
print 'rc                  = ', rc
print 'dt                  = ', integrator.dt
print 'skin                = ', system.skin
print 'temperature         = ', temperature
print 'nsteps              = ', nsteps
print 'isteps              = ', isteps
print 'NodeGrid            = ', system.storage.getNodeGrid()
print 'CellGrid            = ', system.storage.getCellGrid()
print ''

# espressopp.tools.decomp.tuneSkin(system, integrator)

espressopp.tools.analyse.info(system, integrator)
for k in range(nsteps):
  integrator.run(isteps)
  espressopp.tools.analyse.info(system, integrator)
  #filename1 = "config/polymer_melt_%0i.xyz" % k
  #espressopp.tools.writexyz(filename1, system, velocities = True, unfolded = False)
  #unfolded config
  #filename2 = "config_unfolded/polymer_melt_%0i.xyz" % k
  #espressopp.tools.writexyz(filename2, system, velocities = True, unfolded = True)
  #pdbs
  #filename3 = "pdbs/polymer_melt_%0i.pdb" % k
  #espressopp.tools.pdbwrite(filename3, system, molsize=num_particles)
espressopp.tools.analyse.info(system, integrator)

